      subroutine physiq(ngrid,nlayer,nq,
     *            firstcall,lastcall,
     *            pday,ptime,ptimestep,
     *            pplev,pplay,pphi,
     *            pu,pv,pt,pq,
     *            pw,
     *            pdu,pdv,pdt,pdq,pdpsrf,tracerdyn)

      use radinc_h, only : naerkind

      implicit none


!==================================================================
!     
!     Purpose
!     -------
!     Central subroutine for all the physics parameterisations in the
!     universal model. Originally adapted from the Mars LMDZ model.
!
!     The model can be run without or with tracer transport
!     depending on the value of "tracer" in file "callphys.def".
!
!
!   It includes:
!
!      1.  Initialization:
!      1.1 Firstcall initializations
!      1.2 Initialization for every call to physiq
!      1.2.5 Compute mean mass and cp, R and thermal conduction coeff.
!      2. Compute radiative transfer tendencies
!         (longwave and shortwave).
!      4. Vertical diffusion (turbulent mixing):
!      5. Convective adjustment
!      6. Condensation and sublimation of gases (currently just CO2).
!      7.  TRACERS :
!       7a. water and water ice
!       7c. other schemes for tracer transport (lifting, sedimentation)
!       7d. updates (pressure variations, surface budget)
!      9. Surface and sub-surface temperature calculations
!     10. Write outputs :
!           - "startfi", "histfi" if it's time
!           - Saving statistics if "callstats = .true."
!           - Output any needed variables in "diagfi" 
!     10. Diagnostics: mass conservation of tracers, radiative energy balance etc.
!
!   arguments
!   ---------
!
!   input
!   -----
!    ecri                  period (in dynamical timestep) to write output
!    ngrid                 Size of the horizontal grid.
!                          All internal loops are performed on that grid.
!    nlayer                Number of vertical layers.
!    nq                    Number of advected fields
!    firstcall             True at the first call
!    lastcall              True at the last call
!    pday                  Number of days counted from the North. Spring
!                          equinoxe.
!    ptime                 Universal time (0<ptime<1): ptime=0.5 at 12:00 UT
!    ptimestep             timestep (s)
!    pplay(ngrid,nlayer)   Pressure at the middle of the layers (Pa)
!    pplev(ngrid,nlayer+1) intermediate pressure levels (pa)
!    pphi(ngrid,nlayer)    Geopotential at the middle of the layers (m2s-2)
!    pu(ngrid,nlayer)      u component of the wind (ms-1)
!    pv(ngrid,nlayer)      v component of the wind (ms-1)
!    pt(ngrid,nlayer)      Temperature (K)
!    pq(ngrid,nlayer,nq)   Advected fields
!    pudyn(ngrid,nlayer)    \ 
!    pvdyn(ngrid,nlayer)     \ Dynamical temporal derivative for the
!    ptdyn(ngrid,nlayer)     / corresponding variables
!    pqdyn(ngrid,nlayer,nq) /
!    pw(ngrid,?)           vertical velocity
!
!   output
!   ------
!
!    pdu(ngrid,nlayermx)        \
!    pdv(ngrid,nlayermx)         \  Temporal derivative of the corresponding
!   pdt(ngrid,nlayermx)         /  variables due to physical processes.
!    pdq(ngrid,nlayermx)        /
!    pdpsrf(ngrid)             /
!    tracerdyn                 call tracer in dynamical part of GCM ?
!
!
!     Authors
!     -------
!           Frederic Hourdin	15/10/93
!           Francois Forget	1994
!           Christophe Hourdin	02/1997 
!           Subroutine completly rewritten by F.Forget (01/2000)
!           Water ice clouds: Franck Montmessin (update 06/2003)
!           Radiatively active tracers: J.-B. Madeleine (10/2008-06/2009)
!           New correlated-k radiative scheme: R. Wordsworth (2009)
!           Many specifically Martian subroutines removed: R. Wordsworth (2009)       
!     
!==================================================================


c    0.  Declarations :
c    ------------------

#include "dimensions.h"
#include "dimphys.h"
#include "comgeomfi.h"
#include "surfdat.h"
#include "comsoil.h"
#include "comdiurn.h"
#include "callkeys.h"
#include "comcstfi.h"
#include "planete.h"
#include "comsaison.h"
#include "control.h"
#include "comg1d.h"
#include "tracer.h"

#include "watercap.h"

#include "netcdf.inc"



c Arguments :
c -----------

c   inputs:
c   -------
      INTEGER ngrid,nlayer,nq
      REAL ptimestep
      REAL pplev(ngridmx,nlayer+1),pplay(ngridmx,nlayer)
      REAL pphi(ngridmx,nlayer)
      REAL pu(ngridmx,nlayer),pv(ngridmx,nlayer)
      REAL pt(ngridmx,nlayer),pq(ngridmx,nlayer,nq)
      REAL pw(ngridmx,nlayer)        ! pvervel transmitted by dyn3d
      REAL zh(ngridmx,nlayermx)      ! potential temperature (K)
      LOGICAL firstcall,lastcall

      REAL pday
      REAL ptime 
      logical tracerdyn

c   outputs:
c   --------
c     physical tendencies
      REAL pdu(ngridmx,nlayer),pdv(ngridmx,nlayer)
      REAL pdt(ngridmx,nlayer),pdq(ngridmx,nlayer,nq)
      REAL pdpsrf(ngridmx) ! surface pressure tendency


c Local saved variables:
c ----------------------
c     aerosol (dust or ice) extinction optical depth  at reference wavelength 
c     "longrefvis" set in dimradmars.h , for one of the "naerkind"  kind of
c      aerosol optical properties:
c      REAL aerosol(ngridmx,nlayermx,naerkind) 
!     this is now internal to callcorrk and hence no longer needed here

      INTEGER day_ini  ! Initial date of the run (sol since Ls=0) 
      INTEGER icount     ! counter of calls to physiq during the run.
      REAL tsurf(ngridmx)            ! Surface temperature (K)
      REAL tsoil(ngridmx,nsoilmx)    ! sub-surface temperatures (K)
      REAL albedo(ngridmx)           ! Surface albedo

      real albedo0(ngridmx)           ! Surface albedo
      save albedo0

      REAL emis(ngridmx)             ! Thermal IR surface emissivity
      REAL dtrad(ngridmx,nlayermx)   ! Net atm. radiative heating rate (K.s-1)
      REAL fluxrad_sky(ngridmx)      ! rad. flux from sky absorbed by surface (W.m-2)
      REAL fluxrad(ngridmx)          ! Net radiative surface flux (W.m-2)
      REAL capcal(ngridmx)           ! surface heat capacity (J m-2 K-1)
      REAL fluxgrd(ngridmx)          ! surface conduction flux (W.m-2)
      REAL qsurf(ngridmx,nqmx)       ! tracer on surface (e.g. kg.m-2)
      REAL q2(ngridmx,nlayermx+1)    ! Turbulent Kinetic Energy 

      SAVE day_ini, icount
      SAVE tsurf,tsoil
      SAVE albedo,emis, q2
      SAVE capcal,fluxgrd,dtrad,fluxrad,fluxrad_sky,qsurf

      REAL stephan   
      DATA stephan/5.67e-08/  ! Stephan Boltzman constant
      SAVE stephan

c Local variables :
c -----------------

!     aerosol (dust or ice) extinction optical depth  at reference wavelength 
!     for the "naerkind" optically active aerosols:
      REAL aerosol(ngridmx,nlayermx,naerkind) 

      CHARACTER*80 fichier 
      INTEGER l,ig,ierr,igout,iq,i, tapphys
!      INTEGER iqmin                     ! Used if iceparty engaged

      REAL fluxsurf_lw(ngridmx)      ! incident LW (IR) surface flux (W.m-2)
      REAL fluxsurf_sw(ngridmx)      ! incident SW (solar) surface flux (W.m-2)
      REAL fluxtop_lw(ngridmx)       ! Outgoing LW (IR) flux to space (W.m-2)
      REAL fluxtop_sw(ngridmx)       ! Outgoing SW (solar) flux to space (W.m-2)

!     included by RW for equilibration test
      real fluxtop_dn(ngridmx)
      real fluxabs_sw(ngridmx) ! absorbed shortwave radiation
      real fluxdyn(ngridmx)    ! horizontal heat transport by dynamics      

      REAL zls                       ! solar longitude (rad)
      REAL zday                      ! date (time since Ls=0, in martian days)
      REAL zzlay(ngridmx,nlayermx)   ! altitude at the middle of the layers
      REAL zzlev(ngridmx,nlayermx+1) ! altitude at layer boundaries
      REAL latvl1,lonvl1             ! Viking Lander 1 point (for diagnostic)

c     Aerosol effective radius used for radiative transfer (units=meters)
      REAL reffrad(ngridmx,nlayermx,naerkind)

c     Tendencies due to various processes:
      REAL dqsurf(ngridmx,nqmx)
      REAL zdtlw(ngridmx,nlayermx)      ! (K/s)
      REAL zdtsw(ngridmx,nlayermx)      ! (K/s)
      REAL cldtlw(ngridmx,nlayermx)     ! (K/s) LW heating rate for clear areas
      REAL cldtsw(ngridmx,nlayermx)     ! (K/s) SW heating rate for clear areas
      REAL zdtsurf(ngridmx)             ! (K/s)
      REAL zdtlscale(ngridmx,nlayermx)
      REAL zdvdif(ngridmx,nlayermx),zdudif(ngridmx,nlayermx)  ! (m.s-2)
      REAL zdhdif(ngridmx,nlayermx), zdtsdif(ngridmx)         ! (K/s)
      REAL zdvadj(ngridmx,nlayermx),zduadj(ngridmx,nlayermx)  ! (m.s-2)
      REAL zdhadj(ngridmx,nlayermx)                           ! (K/s)
      REAL zdtgw(ngridmx,nlayermx)                            ! (K/s)
      REAL zdugw(ngridmx,nlayermx),zdvgw(ngridmx,nlayermx)    ! (m.s-2)
      REAL zdtc(ngridmx,nlayermx),zdtsurfc(ngridmx)
      REAL zdvc(ngridmx,nlayermx),zduc(ngridmx,nlayermx)

      REAL zdqdif(ngridmx,nlayermx,nqmx), zdqsdif(ngridmx,nqmx)
      REAL zdqsed(ngridmx,nlayermx,nqmx), zdqssed(ngridmx,nqmx)
      REAL zdqdev(ngridmx,nlayermx,nqmx), zdqsdev(ngridmx,nqmx)
      REAL zdqadj(ngridmx,nlayermx,nqmx)
      REAL zdqc(ngridmx,nlayermx,nqmx)
      REAL zdqlscale(ngridmx,nlayermx,nqmx)
      REAL zdqslscale(ngridmx,nqmx)
      REAL zdqchim(ngridmx,nlayermx,nqmx)
      REAL zdqschim(ngridmx,nqmx)

      REAL zdteuv(ngridmx,nlayermx)    ! (K/s)
      REAL zdtconduc(ngridmx,nlayermx) ! (K/s)
      REAL zdumolvis(ngridmx,nlayermx)
      REAL zdvmolvis(ngridmx,nlayermx)
      real zdqmoldiff(ngridmx,nlayermx,nqmx)

c     Local variable for local intermediate calcul:
      REAL zflubid(ngridmx)
      REAL zplanck(ngridmx),zpopsk(ngridmx,nlayermx)
      REAL zdum1(ngridmx,nlayermx)
      REAL zdum2(ngridmx,nlayermx)
      REAL ztim1,ztim2,ztim3, z1,z2
      REAL ztime_fin
      REAL zdh(ngridmx,nlayermx)
      INTEGER length
      PARAMETER (length=100)

c local variables only used for diagnostic (output in file "diagfi" or "stats")
c -----------------------------------------------------------------------------
      REAL ps(ngridmx), zt(ngridmx,nlayermx)
      REAL zu(ngridmx,nlayermx),zv(ngridmx,nlayermx)
      REAL zq(ngridmx,nlayermx,nqmx)
      REAL fluxtop_sw_tot(ngridmx), fluxsurf_sw_tot(ngridmx)
      character*2 str2
      character*5 str5
      real zdtdif(ngridmx,nlayermx), zdtadj(ngridmx,nlayermx)
      real zdtdyn(ngridmx,nlayermx),ztprevious(ngridmx,nlayermx)
      save ztprevious 
      real reff(ngridmx,nlayermx) ! effective dust radius (used if doubleq=T)
      real qtot1,qtot2            ! total aerosol mass
      integer igmin, lmin
      logical tdiag

      real co2col(ngridmx) ! CO2 column
      REAL zplev(ngrid,nlayermx+1),zplay(ngrid,nlayermx)
      REAL zstress(ngrid), cd
      real hco2(nqmx),tmean, zlocal(nlayermx)
      real rho(ngridmx,nlayermx) ! density
      real vmr(ngridmx,nlayermx) ! volume mixing ratio

      REAL time_phys

!     included by RW for kastprof scheme
      logical kastprof

!     reinstated by RW for diagnostic
      REAL tau_col(ngrid)

!     included by RW for tidally locked dynamics
!      logical tlocked

!     included by RW to reduce insanity of code
      real flux1,flux2,flux3,ts1,ts2,ts3

!     included by RW for temporary comparison
      REAL zdtnirco2(ngridmx,nlayermx) ! (K/s)

!     included by RW to compute tracer column densities
      real qcol(ngridmx,nqmx)

!     included by RW for H2O precipitation
      REAL zdtrain(ngridmx,nlayermx)
      REAL zdqrain(ngridmx,nlayermx,nqmx)
      REAL zdqsrain(ngridmx)
      REAL zdqssnow(ngridmx)
      REAL rainout(ngridmx)
      
!     included by RW for H2O Manabe scheme
      REAL dtmoist(ngridmx,nlayermx)
      REAL dqmoist(ngridmx,nlayermx,nqmx)

      real qvap(ngridmx,nlayermx)
      real dqvaplscale(ngridmx,nlayermx)
      real dqcldlscale(ngridmx,nlayermx)
      real cloudfrac(ngridmx,nlayermx)
      real rneb_man(ngridmx,nlayermx)
      real rneb_lsc(ngridmx,nlayermx)

!    included by RW to account for surface cooling by evaporation
      REAL dtsurfh2olat(ngridmx)

!     included by RW to test energy conservation
      REAL dEtot, dEtots, masse, vabs, dvabs

!     included by RW to test water conservation
      real h2otot

!     included by RW to allow variations in cp with location
      REAL cpp3D(ngridmx,nlayermx)   ! specific heat capacity at const. pressure
      REAL rcp3D(ngridmx,nlayermx)   ! R / specific heat capacity
      real cppNI, rcpNI, nnu ! last one just for Seb version
      REAL zpopskNI(ngridmx,nlayermx)

!     included by RW to make 1D saves not every timestep
      integer countG1D
      save countG1D

c=======================================================================

      kastprof=.false.

c 1. Initialisation:
c -----------------

c  1.1   Initialisation only at first call
c  ---------------------------------------
      IF (firstcall) THEN

         if(ngrid.eq.1)then
            !saveG1D=day_step*10
            saveG1D=1
            countG1D=1
         endif

c        variables set to 0
c        ~~~~~~~~~~~~~~~~~~
         call zerophys(ngrid*nlayer,dtrad)
         call zerophys(ngrid,fluxrad)

c        initialize tracer names, indexes and properties
c        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         tracerdyn=tracer
         IF (tracer) THEN
            CALL initracer()
         ENDIF  ! end tracer


c        read startfi (initial parameters)
c        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         
         CALL phyetat0 ("startfi.nc",0,0,
     &         nsoilmx,nq,
     &         day_ini,time_phys,
     &         tsurf,tsoil,emis,q2,qsurf)

         if (pday.ne.day_ini) then
           write(*,*) "PHYSIQ: ERROR: bad synchronization between ",
     &                "physics and dynamics"
           write(*,*) "dynamics day: ",pday
           write(*,*) "physics day:  ",day_ini
           stop
         endif

         write (*,*) 'In physiq day_ini =', day_ini


c        Initialize albedo and orbital calculation
c        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         CALL surfini(ngrid,qsurf,albedo)
! surely for good CO2 ice feedback this must be called every turn?
         CALL iniorbit(aphelie,periheli,year_day,peri_day,obliquit)


         if(tlocked)then
            print*,'Planet is tidally locked at resonance n=',nres
            print*,'Make sure you have the right rotation rate!!!'
         endif


c        initialize soil 
c        ~~~~~~~~~~~~~~~
         IF (callsoil) THEN
            CALL soil(ngrid,nsoilmx,firstcall,inertiedat,
     s          ptimestep,tsurf,tsoil,capcal,fluxgrd)
         ELSE
            PRINT*,'WARNING! Thermal conduction in the soil turned off'
            DO ig=1,ngrid
               capcal(ig)=1.e6
               fluxgrd(ig)=0.
            ENDDO
         ENDIF
         icount=1

c        Set temperature just above condensation temperature (for Early Mars)
c        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         if(nearco2cond) then
            write(*,*)' WARNING, starting at Tcond+1K !!!'
            DO l=1, nlayer
               DO ig=1,ngrid
                  pdt(ig,l)= ((-3167.8)/(log(.01*pplay(ig,l))-23.23)+4
     &                 -pt(ig,l)) / ptimestep
               ENDDO
            ENDDO
         endif

         if(meanOLR)then
            ! to record global radiative balance
            call system('rm -f rad_bal.out')
         endif

         if(water)then
            ! initialise variables for water cycle

            if(ngrid.eq.1)then
               qsurf(1,igcm_h2o_ice) = 100.0
               DO l=1, nlayer
                  pq(1,l,igcm_h2o_ice)=0.0!1.0e-2 
               enddo
            endif

         endif
         call su_watercycle ! even if we don't have a water cycle, we might
                            ! need epsi for the wvp definitions in callcorrk.F

      ENDIF        !  (end of "if firstcall")

c ---------------------------------------------------
c 1.2   Initializations done at every physical timestep:
c ---------------------------------------------------
c
      IF (ngrid.NE.ngridmx) THEN
         PRINT*,'STOP in PHYSIQ'
         PRINT*,'Probleme de dimensions :'
         PRINT*,'ngrid     = ',ngrid
         PRINT*,'ngridmx   = ',ngridmx
         STOP
      ENDIF

c     Initialize various variables
c     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      call zerophys(ngrid*nlayer, pdv)
      call zerophys(ngrid*nlayer, pdu)
      if ( (.not.nearco2cond).and.(.not.firstcall) ) then
         call zerophys(ngrid*nlayer,pdt)
      endif ! this was the source of an evil bug...
      call zerophys(ngrid*nlayer*nq, pdq)
      call zerophys(ngrid, pdpsrf)
      call zerophys(ngrid, zflubid)
      call zerophys(ngrid, zdtsurf)
      call zerophys(ngrid*nq, dqsurf)
      igout=ngrid/2+1 

      zday=pday+ptime ! compute time, in sols (and fraction thereof)

c     Compute Solar Longitude (Ls) :
c     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      if (season) then
         call solarlong(zday,zls)
      else
         call solarlong(float(day_ini),zls)
      end if

c     Compute geopotential between layers
c     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      DO l=1,nlayer
         DO ig=1,ngrid
            zzlay(ig,l)=pphi(ig,l)/g
         ENDDO
      ENDDO
      DO ig=1,ngrid
         zzlev(ig,1)=0.
         zzlev(ig,nlayer+1)=1.e7    ! dummy top of last layer above 10000 km...
      ENDDO
      DO l=2,nlayer
         DO ig=1,ngrid
            z1=(pplay(ig,l-1)+pplev(ig,l))/(pplay(ig,l-1)-pplev(ig,l))
            z2=(pplev(ig,l)+pplay(ig,l))/(pplev(ig,l)-pplay(ig,l))
            zzlev(ig,l)=(z1*zzlay(ig,l-1)+z2*zzlay(ig,l))/(z1+z2)
         ENDDO
      ENDDO

!     Potential temperature calculation not the same in physiq and dynamic...

c     Compute potential temperature
c     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      if(nonideal)then

         DO l=1,nlayer
            DO ig=1,ngrid 
               call calc_cpp3d(cppNI,rcpNI,pt(ig,l),pplay(ig,l))
               cpp3D(ig,l)  = cppNI
               rcp3D(ig,l)  = rcpNI
            ENDDO
         ENDDO

         DO l=1,nlayer
            DO ig=1,ngrid

!               nnu=0.35
!               zh(ig,l)   = (pt(ig,l)**nnu +
!     &              nnu*(460**nnu)*(8.314/1000.)*
!     &           log(pplev(ig,1)/pplay(ig,l)) )**(-nnu) ! Sebastian's version
!               zpopskNI(ig,l) = (pplay(ig,l)/pplev(ig,1))**rcp3D(ig,l)

               zh(ig,l)       = pt(ig,l)*1.0024 - 0.659
               zpopskNI(ig,l) = pt(ig,l)/zh(ig,l)
               ! we're only after zpopskNI here, not zh
               ! zh is calculated seperately before both vdifc and convadj
            ENDDO
         ENDDO

      endif
!      else
         
      DO l=1,nlayer
         DO ig=1,ngrid 
            zpopsk(ig,l)=(pplay(ig,l)/pplev(ig,1))**rcp
            zh(ig,l)=pt(ig,l)/zpopsk(ig,l)
         ENDDO
      ENDDO

!      endif


c-----------------------------------------------------------------------
c    2. Compute radiative tendencies :
c-----------------------------------------------------------------------

      IF (callrad) THEN
         IF( MOD(icount-1,iradia).EQ.0) THEN

c          Local stellar zenith angle
c          ~~~~~~~~~~~~~~~~~~~~~~~~~~
           CALL orbite(zls,dist_sol,declin)

           IF (tlocked) THEN
              ztim1=SIN(declin)
              ztim2=COS(declin)*COS(2.*pi*(zday/year_day) - zls*nres)
              ztim3=-COS(declin)*SIN(2.*pi*(zday/year_day) - zls*nres)

               CALL solang(ngrid,sinlon,coslon,sinlat,coslat,
     s         ztim1,ztim2,ztim3,mu0,fract)

           ELSEIF (diurnal) THEN
               ztim1=SIN(declin)
               ztim2=COS(declin)*COS(2.*pi*(zday-.5))
               ztim3=-COS(declin)*SIN(2.*pi*(zday-.5))

               CALL solang(ngrid,sinlon,coslon,sinlat,coslat,
     s         ztim1,ztim2,ztim3,mu0,fract)

           ELSE

               CALL mucorr(ngrid,declin,lati,mu0,fract,10000.,rad)
               ! WARNING: this function appears not to work in 1D

           ENDIF

c          Call main radiative transfer scheme
c          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

c          Radiative transfer
c          ------------------

           if (corrk) then
              if(kastprof)then
                 call kastprof_fn(tsurf,pt,pplay,pplev)
                 print*,'pt',pt
                 print*,'pplay',pplay
              endif

              CALL callcorrk(icount,ngrid,nlayer,pq,nq,qsurf,
     &             albedo,emis,mu0,pplev,pplay,pt,
     &             tsurf,fract,dist_sol,igout,aerosol,cpp3D,
     &             zdtlw,zdtsw,fluxsurf_lw,fluxsurf_sw,fluxtop_lw,
     &             fluxtop_sw,fluxtop_dn,reffrad,tau_col,ptime,pday,
     &             firstcall,lastcall)
            


c          Radiative flux from the sky absorbed by the surface (W.m-2)
c          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
              DO ig=1,ngrid
                 fluxrad_sky(ig)=emis(ig)*fluxsurf_lw(ig)
     &                +fluxsurf_sw(ig)*(1.-albedo(ig))
              ENDDO

c          Net atmospheric radiative heating rate (K.s-1)
c          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

              DO l=1,nlayer
                 DO ig=1,ngrid
                    dtrad(ig,l)=zdtsw(ig,l)+zdtlw(ig,l)
                 ENDDO
              ENDDO

           else

c          Atmosphere has no radiative effect
c          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
              DO ig=1,ngrid
                 fluxtop_dn(ig)  = fract(ig)*mu0(ig)*Fat1AU/dist_sol**2
                 fluxrad_sky(ig) = fluxtop_dn(ig)*(1.-albedo(ig))
                 fluxtop_sw(ig)  = fluxtop_dn(ig)*albedo(ig)
                 fluxtop_lw(ig)  = emis(ig)*stephan*tsurf(ig)**4
              ENDDO ! radiation skips the atmosphere entirely


              DO l=1,nlayer
                 DO ig=1,ngrid
                    dtrad(ig,l)=0.0
                 ENDDO
              ENDDO ! hence no atmospheric radiative heating

           endif                ! if corrk

        ENDIF ! of if(mod(icount-1,iradia).eq.0)

!    Transformation of the radiative tendencies:
!    -------------------------------------------

!          Net radiative surface flux (W.m-2)
!          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        DO ig=1,ngrid
           zplanck(ig)=tsurf(ig)*tsurf(ig)
           zplanck(ig)=emis(ig)*
     &          stephan*zplanck(ig)*zplanck(ig)
           fluxrad(ig)=fluxrad_sky(ig)-zplanck(ig)
        ENDDO

        DO l=1,nlayer
           DO ig=1,ngrid
              pdt(ig,l)=pdt(ig,l)+dtrad(ig,l)
           ENDDO
        ENDDO

!-------------------------
! test energy conservation
         if(enertest)then
            dEtot=0.0
            dEtots=0.0
            DO ig = 1, ngrid
               DO l = 1, nlayer
                  masse = (pplev(ig,l) - pplev(ig,l+1))/g
                  dEtot = dEtot + cpp*masse*dtrad(ig,l)*area(ig)
                  !print*,'l=',l,'dEtot=',cpp*masse*dtrad(ig,l)*area(ig)
               ENDDO
               dEtots = dEtots + fluxrad(ig)*area(ig)
            ENDDO
            dEtot=dEtot/totarea
            dEtots=dEtots/totarea
            print*,'In corrk atmospheric energy change=',dEtot,' W m-2'
            print*,'In corrk surface energy change=',dEtots,' W m-2'
         endif
!-------------------------

      ENDIF ! of IF (callrad)


!-----------------------------------------------------------------------
!    4. Vertical diffusion (turbulent mixing):
!    -----------------------------------------

      IF (calldifv) THEN

         DO ig=1,ngrid
            zflubid(ig)=fluxrad(ig)+fluxgrd(ig)
         ENDDO

         CALL zerophys(ngrid*nlayer,zdum1)
         CALL zerophys(ngrid*nlayer,zdum2)
         do l=1,nlayer
            do ig=1,ngrid
               zdh(ig,l)=pdt(ig,l)/zpopsk(ig,l)
            enddo
         enddo
         
c        Calling vdif (Martian version WITH CO2 condensation)
         CALL vdifc(ngrid,nlayer,nq,zpopsk,
     &        ptimestep,capcal,lwrite,
     &        pplay,pplev,zzlay,zzlev,z0,
     &        pu,pv,zh,pq,tsurf,emis,qsurf,
     &        zdum1,zdum2,zdh,pdq,zflubid,
     &        zdudif,zdvdif,zdhdif,zdtsdif,q2,
     &        zdqdif,zdqsdif)

         DO l=1,nlayer
            DO ig=1,ngrid
               pdv(ig,l)=pdv(ig,l)+zdvdif(ig,l)
               pdu(ig,l)=pdu(ig,l)+zdudif(ig,l)
               pdt(ig,l)=pdt(ig,l)+zdhdif(ig,l)*zpopsk(ig,l)

               zdtdif(ig,l)=zdhdif(ig,l)*zpopsk(ig,l) ! for diagnostic only
            ENDDO
         ENDDO

         DO ig=1,ngrid
            zdtsurf(ig)=zdtsurf(ig)+zdtsdif(ig)
         ENDDO

         if (tracer) then 
           DO iq=1, nq
            DO l=1,nlayer
              DO ig=1,ngrid
                 pdq(ig,l,iq)=pdq(ig,l,iq)+ zdqdif(ig,l,iq) 
              ENDDO
            ENDDO
           ENDDO
           DO iq=1, nq
              DO ig=1,ngrid
                 dqsurf(ig,iq)=dqsurf(ig,iq) + zdqsdif(ig,iq)
              ENDDO
           ENDDO

         end if ! of if (tracer)

         !print*,'surface temp loss:'
         !print*, dtsurfh2olat

!-------------------------
! test energy conservation
         if(enertest)then
            dEtot=0.0
            dEtots=0.0
            DO ig = 1, ngrid
               DO l = 1, nlayer
                  masse = (pplev(ig,l) - pplev(ig,l+1))/g

                  dEtot = dEtot + cpp*masse*zdtdif(ig,l)*area(ig)
                  
                  vabs  = sqrt(pdu(ig,l)**2 + pdv(ig,l)**2)
                  dvabs = sqrt(zdudif(ig,l)**2 + zdvdif(ig,l)**2)
                  dEtot = dEtot + masse*vabs*dvabs*area(ig)
               ENDDO
               dEtots = dEtots + capcal(ig)*zdtsdif(ig)*area(ig)
            ENDDO
            dEtot=dEtot/totarea
            dEtots=dEtots/totarea
            print*,'In difv atmospheric energy change=',dEtot,' W m-2'
            print*,'In difv surface energy change=',dEtots,' W m-2'
         endif
!-------------------------

      ELSE    

         DO ig=1,ngrid
            zdtsurf(ig)=zdtsurf(ig)+
     s      (fluxrad(ig)+fluxgrd(ig))/capcal(ig)
         ENDDO

      ENDIF ! of IF (calldifv)


!-----------------------------------------------------------------------
!   5. Dry convective adjustment:
!   -----------------------------

      IF(calladj) THEN

         DO l=1,nlayer
            DO ig=1,ngrid
               if(nonideal)then
                  zdh(ig,l)=pdt(ig,l)/zpopskNI(ig,l)
               else
                  zdh(ig,l)=pdt(ig,l)/zpopsk(ig,l)
               endif
            ENDDO
         ENDDO
         CALL zerophys(ngrid*nlayer,zduadj)
         CALL zerophys(ngrid*nlayer,zdvadj)
         CALL zerophys(ngrid*nlayer,zdhadj)

         if(nonideal)then
            CALL convadj(ngrid,nlayer,nq,ptimestep,
     &           pplay,pplev,zpopskNI,
     &           pu,pv,zh,pq,
     &           pdu,pdv,zdh,pdq,
     &           zduadj,zdvadj,zdhadj,
     &           zdqadj)
         else
            CALL convadj(ngrid,nlayer,nq,ptimestep,
     &           pplay,pplev,zpopsk,
     &           pu,pv,zh,pq,
     &           pdu,pdv,zdh,pdq,
     &           zduadj,zdvadj,zdhadj,
     &           zdqadj)
         endif


         DO l=1,nlayer
            DO ig=1,ngrid
               pdu(ig,l)=pdu(ig,l)+zduadj(ig,l)
               pdv(ig,l)=pdv(ig,l)+zdvadj(ig,l)
               if(nonideal)then
                  pdt(ig,l)=pdt(ig,l)+zdhadj(ig,l)*zpopskNI(ig,l)
                  zdtadj(ig,l)=zdhadj(ig,l)*zpopskNI(ig,l) ! for diagnostic only
               else
                  pdt(ig,l)=pdt(ig,l)+zdhadj(ig,l)*zpopsk(ig,l)
                  zdtadj(ig,l)=zdhadj(ig,l)*zpopsk(ig,l) ! for diagnostic only
               endif
            ENDDO
         ENDDO

         if(tracer) then 
           DO iq=1, nq
            DO l=1,nlayer
              DO ig=1,ngrid
                 pdq(ig,l,iq)=pdq(ig,l,iq)+ zdqadj(ig,l,iq) 
              ENDDO
            ENDDO
           ENDDO
         end if

!-------------------------
! test energy conservation
         if(enertest)then
            dEtot=0.0
            DO ig = 1, ngrid
               DO l = 1, nlayer
                  masse = (pplev(ig,l) - pplev(ig,l+1))/g
                  dEtot = dEtot + cpp*masse*zdtadj(ig,l)*area(ig)
               ENDDO
            ENDDO
            dEtot=dEtot/totarea
          print*,'In convadj atmospheric energy change=',dEtot,' W m-2'
         endif
!-------------------------

      ENDIF ! of IF(calladj)

!-----------------------------------------------------------------------
!   6. Carbon dioxide condensation-sublimation:
!   -------------------------------------------

      IF (co2cond) THEN
         if (.not.tracer) then
            print*,'We need a CO2 ice tracer to condense CO2'
            call abort
         endif

         call condens_co2cloud(ngrid,nlayer,nq,ptimestep,
     &        capcal,pplay,pplev,tsurf,pt,
     &        pphi,pdt,pdu,pdv,zdtsurf,pu,pv,pq,pdq,
     &        qsurf(1,igcm_co2_ice),albedo,emis,
     &        zdtc,zdtsurfc,pdpsrf,zduc,zdvc,
     &        zdqc,reffrad,cpp3D)

         DO l=1,nlayer
           DO ig=1,ngrid
             pdt(ig,l)=pdt(ig,l)+zdtc(ig,l)
             pdv(ig,l)=pdv(ig,l)+zdvc(ig,l)
             pdu(ig,l)=pdu(ig,l)+zduc(ig,l)
           ENDDO
         ENDDO
         DO ig=1,ngrid
            zdtsurf(ig) = zdtsurf(ig) + zdtsurfc(ig)
         ENDDO

!         IF (tracer) THEN
           DO iq=1, nq ! should use new notation here !
            DO l=1,nlayer
              DO ig=1,ngrid
                pdq(ig,l,iq)=pdq(ig,l,iq)+ zdqc(ig,l,iq) 
              ENDDO
            ENDDO
           ENDDO
           !NB: we do not add tendency on surface co2ice ; 
           ! since qsurf(:,igcm_co2_ice) is updated in condens_co2cloud
!         ENDIF ! of IF (tracer)


!-------------------------
! test energy conservation
         if(enertest)then
            dEtot=0.0
            DO ig = 1, ngrid
               DO l = 1, nlayer
                  masse = (pplev(ig,l) - pplev(ig,l+1))/g
                  dEtot = dEtot + cpp*masse*zdtc(ig,l)
               ENDDO
               dEtot = dEtot + capcal(ig)*zdtsurfc(ig)
            ENDDO
            print*,'In co2cloud energy change=',dEtot,' W m-2'
         endif ! we're missing the latent heat...
!-------------------------

      ENDIF  ! of IF (co2cond)



c-----------------------------------------------------------------------
c   7. Specific parameterizations for tracers 
c   -----------------------------------------

      if (tracer) then 

c   7a. Water and ice
c     ---------------

c        ---------------------------------------
c        Water ice condensation in the atmosphere
c        ----------------------------------------
         IF (water) THEN

            if(watercond)then

               if(1.eq.2)then
                  call moistadj(pt,pq,pplev,pplay,dtmoist,dqmoist,
     &                 ptimestep,rneb_man)

                  DO l=1,nlayer
                     DO ig=1,ngrid
                        pdq(ig,l,igcm_h2o_vap)=pdq(ig,l,igcm_h2o_vap)+
     &                       dqmoist(ig,l,igcm_h2o_vap)
                        pdq(ig,l,igcm_h2o_ice)=pdq(ig,l,igcm_h2o_ice)+
     &                       dqmoist(ig,l,igcm_h2o_ice)
                        pdt(ig,l)=pdt(ig,l)+dtmoist(ig,l)
                     ENDDO
                  ENDDO
               endif            ! moistadj turned off for now

              DO l=1,nlayer
                 DO ig=1,ngrid
                     qvap(ig,l)=pq(ig,l,igcm_h2o_vap)
                 ENDDO
              ENDDO
              call largescale2(ptimestep,pplev,pplay,pt,qvap,
     s             zdtlscale, dqvaplscale,dqcldlscale,rneb_lsc)
              DO l=1,nlayer
                 DO ig=1,ngrid
                    pdq(ig,l,igcm_h2o_vap)=pdq(ig,l,igcm_h2o_vap)+
     &                   dqvaplscale(ig,l)
                    pdq(ig,l,igcm_h2o_ice)=pdq(ig,l,igcm_h2o_ice)+
     &                   dqcldlscale(ig,l)
                !    pdt(ig,l)=pdt(ig,l)+zdtlscale(ig,l)
                    ! for the moment this is removed, as it dont conserve energy
                 ENDDO
              ENDDO

              ! compute cloud fraction
              DO l = 1, nlayer
                 DO ig = 1,ngrid
                    cloudfrac(ig,l)=MAX(rneb_lsc(ig,l),rneb_man(ig,l)) 
                 ENDDO
              ENDDO

           endif                ! of IF (watercondense)
           
           if(waterrain)then

              call rain(ptimestep,pplev,pplay,pt,pdt,pq,pdq,
     &             zdtrain,zdqrain,zdqsrain,zdqssnow,cloudfrac)

              DO l=1,nlayer
                 DO ig=1,ngrid
                    pdq(ig,l,igcm_h2o_vap)=pdq(ig,l,igcm_h2o_vap)+
     &                   zdqrain(ig,l,igcm_h2o_vap)
                    pdq(ig,l,igcm_h2o_ice)=pdq(ig,l,igcm_h2o_ice)+
     &                   zdqrain(ig,l,igcm_h2o_ice)
                    !pdt(ig,l)=pdt(ig,l)+zdtrain(ig,l)
                 ENDDO
              ENDDO

              DO ig=1,ngrid
                 dqsurf(ig,igcm_h2o_ice)=dqsurf(ig,igcm_h2o_ice)+
     &                zdqsrain(ig)+zdqssnow(ig)
                 rainout(ig)=zdqsrain(ig)+zdqssnow(ig) ! diagnostic   
                 !rainout(ig)=zdqssnow(ig)
              ENDDO

         end if                 ! of IF (waterrain)

      end if                    ! of IF (water)


c   7c. Aerosol particles
c     -------------------

c        ------------- 
c        Sedimentation
c        ------------- 
         IF (sedimentation) THEN 
           call zerophys(ngrid*nlayer*nq, zdqsed)
           call zerophys(ngrid*nq, zdqssed)

           call callsedim(ngrid,nlayer, ptimestep,
     &          pplev,zzlev, pt,
     &          pq, pdq, zdqsed, zdqssed,nq)

           DO iq=2, nq ! should be updated
             DO l=1,nlayer
               DO ig=1,ngrid
                    pdq(ig,l,iq)=pdq(ig,l,iq)+ zdqsed(ig,l,iq)
               ENDDO
             ENDDO
           ENDDO
           DO iq=2, nq
             DO ig=1,ngrid
                dqsurf(ig,iq)= dqsurf(ig,iq) + zdqssed(ig,iq)
             ENDDO
           ENDDO
         END IF   ! of IF (sedimentation)


c   7d. Updates
c     ---------

        DO iq=1, nq
          DO ig=1,ngrid

c       ---------------------------------
c       Updating tracer budget on surface
c       ---------------------------------
            qsurf(ig,iq)=qsurf(ig,iq)+ptimestep*dqsurf(ig,iq)

          ENDDO  ! (ig)
        ENDDO    ! (iq)

      endif !  of if (tracer) 


!-----------------------------------------------------------------------
!   9. Surface and sub-surface soil temperature
!-----------------------------------------------------------------------
!
!
!   9.1 Increment Surface temperature:
!   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      DO ig=1,ngrid
         tsurf(ig)=tsurf(ig)+ptimestep*zdtsurf(ig) 
      ENDDO

!
!   9.2 Compute soil temperatures and subsurface heat flux:
!   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      IF (callsoil) THEN
         CALL soil(ngrid,nsoilmx,.false.,inertiedat,
     &          ptimestep,tsurf,tsoil,capcal,fluxgrd)
      ENDIF

!-------------------------
! test energy conservation
         if(enertest)then
            dEtots=0.0
            DO ig = 1, ngrid
               dEtots = dEtots + capcal(ig)*zdtsurf(ig)*area(ig)
            ENDDO
            dEtots=dEtots/totarea
            print*,'Surface energy change=',dEtots,' W m-2'
         endif
!-------------------------




!-----------------------------------------------------------------------
!  10. Perform diagnostics and write output files
!-----------------------------------------------------------------------

!     ---------------------------------------------------------
!     Check the energy balance of the simulation during the run
!     ---------------------------------------------------------
      flux1 = 0
      flux2 = 0
      flux3 = 0
      do ig=1,ngrid

	    flux1 = flux1 + 
     &        area(ig)*(fluxtop_dn(ig) - fluxtop_sw(ig))
	    flux2 = flux2 + area(ig)*fluxtop_lw(ig) 
	    flux3 = flux3 + area(ig)*fluxtop_dn(ig) 
            fluxabs_sw(ig)=fluxtop_dn(ig) - fluxtop_sw(ig)

      end do
      print*,'Incident solar flux, absorbed solar flux, OLR (W/m2)'
      print*, flux3/totarea, '      ',  flux1/totarea ,
     & '           = ', flux2/totarea 

      if(meanOLR)then
         ! to record global radiative balance
         open(92,file="rad_bal.out",form='formatted',access='append')
         write(92,*) zday,flux3/totarea,flux1/totarea,flux2/totarea
         close(92)
      endif

      if(kastprof)then ! for f(p,T) computation (not needed in universal model)
         open(15,file='ene_bal.out',form='formatted',access='append')
         write(15,*) flux1
         close(15)
         stop
      endif

!     Surface temperature information
      ts1 = 0
      ts2 = 999
      ts3 = 0
      do ig=1,ngrid
         ts1 = ts1 + area(ig)*tsurf(ig)
         ts2 = min(ts2,tsurf(ig))  
	 ts3 = max(ts3,tsurf(ig))  
      end do
      print*,'Mean Tsurf =',ts1/totarea , ' Min Tsurf=',ts2, 
     &          ' Max Tsurf =',ts3

!    -------------------------------
!    Dynamical fields incrementation
!    -------------------------------
! (FOR OUTPUT ONLY : the actual model integration is performed in the dynamics)
      ! temperature, zonal and meridional wind
      DO l=1,nlayer
        DO ig=1,ngrid
          zt(ig,l)=pt(ig,l) + pdt(ig,l)*ptimestep
          zu(ig,l)=pu(ig,l) + pdu(ig,l)*ptimestep
          zv(ig,l)=pv(ig,l) + pdv(ig,l)*ptimestep

      ! diagnostic
            zdtdyn(ig,l)=ztprevious(ig,l)-pt(ig,l)
            ztprevious(ig,l)=zt(ig,l)
        ENDDO
      ENDDO
      if(firstcall)call zerophys(zdtdyn)

      ! dynamical heating diagnostic
      DO ig=1,ngrid
         fluxdyn(ig)=0.
         DO l=1,nlayer
            if(nonideal)then
               fluxdyn(ig)=fluxdyn(ig) - (zdtdyn(ig,l)/ptimestep)
     &              *(pplev(ig,l)-pplev(ig,l+1))*cpp3D(ig,l)/g
            else
               fluxdyn(ig)=fluxdyn(ig) - (zdtdyn(ig,l)/ptimestep)
     &              *(pplev(ig,l)-pplev(ig,l+1))*cpp/g
            endif
         ENDDO
      ENDDO


      ! tracers
      DO iq=1, nq
        DO l=1,nlayer
          DO ig=1,ngrid
            zq(ig,l,iq)=pq(ig,l,iq) +pdq(ig,l,iq)*ptimestep
          ENDDO
        ENDDO
      ENDDO

      ! surface pressure
      DO ig=1,ngrid
          ps(ig)=pplev(ig,1) + pdpsrf(ig)*ptimestep
      ENDDO

      ! pressure
      DO l=1,nlayer
        DO ig=1,ngrid
             zplev(ig,l)=pplev(ig,l)/pplev(ig,1)*ps(ig)
             zplay(ig,l)=pplay(ig,l)/pplev(ig,1)*ps(ig)
        ENDDO
      ENDDO

    
!     ---------------------------------------------------------
!     Compute column amounts (kg m-2) if tracers are enabled
!     ---------------------------------------------------------
      if(tracer)then
         call zerophys(ngrid*nq,qcol)
         do iq=1,nq
            DO ig=1,ngrid
               DO l=1,nlayer
                  qcol(ig,iq) = qcol(ig,iq) + zq(ig,l,iq) *
     &                      (pplev(ig,l) - pplev(ig,l+1)) / g
               enddo
            enddo
         enddo
      endif


!     ---------------------------------------------------------
!     Test for water conservation if water is enabled
!     ---------------------------------------------------------
      if(water)then

         h2otot = 0
         do ig=1,ngrid
            h2otot = h2otot + area(ig)*
     &      (qcol(ig,igcm_h2o_ice)+qcol(ig,igcm_h2o_vap)
     &      +qsurf(ig,igcm_h2o_ice)+qsurf(ig,igcm_h2o_vap))
         end do
         print*,'Total water amount (kg): ',h2otot!/totarea
      endif   


!     TEMPORARY: save water cycle diagnostics
!         IF((ngrid.eq.1).and.lastcall) THEN
!            open(13,file='Ts.out')
!            open(14,file='T.out')
!            open(15,file='p.out')
!            open(16,file='vap.out')
!            open(17,file='alt.out')
!            ig=1
!            write(13,*) tsurf(ig)
!            DO l=1,nlayer
!               write(14,*) pt(ig,l)
!              write(15,*) pplay(ig,l)
!               write(16,*) pq(ig,l,igcm_h2o_vap)
!               write(17,*) zzlay(ig,l)
!            enddo
!            close(13)
!            close(14)
!            close(15)
!            close(16)
!            close(17)
!         ENDIF


!     TEMPORARY: save data for LBL
!         IF((ngrid.eq.1).and.lastcall) THEN
!            open(14,file='T.dat')
!            open(15,file='p.dat')
!            ig=1
!            write(14,*) nlayer+1
!            write(15,*) nlayer+1
!            write(14,*) tsurf(ig)
!            write(15,*) pplev(ig,1)
!            DO l=1,nlayer
!               write(14,*) pt(ig,l)
!              write(15,*) pplay(ig,l)
!            enddo
!            close(14)
!            close(15)
!         ENDIF


      IF (ngrid.NE.1) THEN
         print*,'Ls =',zls*180./pi

!        -------------------------------------------------------------------
!        Writing NetCDF file  "RESTARTFI" at the end of the run
!        -------------------------------------------------------------------
!        Note: 'restartfi' is stored just before dynamics are stored
!              in 'restart'. Between now and the writting of 'restart',
!              there will have been the itau=itau+1 instruction and
!              a reset of 'time' (lastacll = .true. when itau+1= itaufin)
!              thus we store for time=time+dtvr

         IF(lastcall) THEN
            ztime_fin = ptime + ptimestep/(float(iphysiq)*daysec)

            write(*,*)'PHYSIQ: for physdem ztime_fin =',ztime_fin
            call physdem1("restartfi.nc",long,lati,nsoilmx,nq,
     .              ptimestep,pday,
     .              ztime_fin,tsurf,tsoil,emis,q2,qsurf,
     .              area,albedodat,inertiedat,zmea,zstd,zsig,
     .              zgam,zthe)
         ENDIF

!        -----------------------------------------------------------------
!        Saving statistics :
!        -----------------------------------------------------------------
!        ("stats" stores and accumulates 8 key variables in file "stats.nc"
!        which can later be used to make the statistic files of the run:
!        "stats")          only possible in 3D runs !

         
         IF (callstats) THEN

            call wstats(ngrid,"ps","Surface pressure","Pa",2,ps)
            call wstats(ngrid,"tsurf","Surface temperature","K",2,tsurf)
             call wstats(ngrid,"fluxsurf_lw",
     .                 "Thermal IR radiative flux to surface","W.m-2",2,
     .                 fluxsurf_lw)
             call wstats(ngrid,"fluxsurf_sw",
     .                  "Solar radiative flux to surface","W.m-2",2,
     .                   fluxsurf_sw_tot)
             call wstats(ngrid,"fluxtop_lw",
     .                  "Thermal IR radiative flux to space","W.m-2",2,
     .                  fluxtop_lw)
             call wstats(ngrid,"fluxtop_sw",
     .                  "Solar radiative flux to space","W.m-2",2,
     .                  fluxtop_sw_tot)

            call wstats(ngrid,"temp","Atmospheric temperature","K",3,zt)
            call wstats(ngrid,"u","Zonal (East-West) wind","m.s-1",3,zu)
            call wstats(ngrid,"v","Meridional (North-South) wind",
     .                  "m.s-1",3,zv)
            call wstats(ngrid,"w","Vertical (down-up) wind",
     .                  "m.s-1",3,pw)
             call wstats(ngrid,"q2",
     .               "Boundary layer eddy kinetic energy","m2.s-2",3,q2)

           if (tracer) then
            if (water) then
               vmr=zq(1:ngridmx,1:nlayermx,igcm_h2o_vap)
     &                  *mugaz/mmol(igcm_h2o_vap)
               call wstats(ngrid,"vmr_h2ovapor",
     .                    "H2O vapor volume mixing ratio","mol/mol",
     .                    3,vmr)
            endif ! of if (water)

           endif !tracer 

            IF(lastcall) THEN
              write (*,*) "Writing stats..."
              call mkstats(ierr)
            ENDIF
          ENDIF !if callstats



!        ----------------------------------------------------------------------
!        output in netcdf file "DIAGFI", containing any variable for diagnostic
!        (output with  period "ecritphy", set in "run.def")
!        ----------------------------------------------------------------------
!        WRITEDIAGFI can ALSO be called from any other subroutine
!        for any variable!

!     is it not preferable to keep all the calls in one place?

        call WRITEDIAGFI(ngrid,"tsurf","Surface temperature","K",2,
     &                  tsurf)
        call WRITEDIAGFI(ngrid,"ps","surface pressure","Pa",2,ps)
        call WRITEDIAGFI(ngrid,"temp","temperature","K",3,zt)
!        call WRITEDIAGFI(ngrid,"tau","tau"," ",2,tau)
!        call WRITEDIAGFI(ngrid,"u","Zonal wind","m.s-1",3,zu)
!        call WRITEDIAGFI(ngrid,"v","Meridional wind","m.s-1",3,zv)
!        call WRITEDIAGFI(ngrid,"w","Vertical wind","m.s-1",3,pw)
!        call WRITEDIAGFI(ngrid,"rho","density","none",3,rho)
!        call WRITEDIAGFI(ngrid,"q2","q2","kg.m-3",3,q2)
!        call WRITEDIAGFI(ngridm,'Teta','T potentielle','K',3,zh)
!        call WRITEDIAGFI(ngridm,'Pression','Pression','Pa',3,pplay)

!     Total energy balance diagnostics
        call WRITEDIAGFI(ngrid,"ISR","incoming stellar rad.","W m-2",
     &       2,fluxtop_dn)
        call WRITEDIAGFI(ngrid,"ASR","absorbed stellar rad.","W m-2",
     &       2,fluxabs_sw)
        call WRITEDIAGFI(ngrid,"OLR","outgoing longwave rad.","W m-2",
     &       2,fluxtop_lw)
        call WRITEDIAGFI(ngrid,"DYN","dynamical heat input","W m-2",
     &       2,fluxdyn)

!     Temporary inclusions for heating diagnostics
!        call WRITEDIAGFI(ngrid,"zdtdyn","Dyn. heating","T s-1",3,zdtdyn)
!        call WRITEDIAGFI(ngrid,"zdtsw","SW heating","T s-1",3,zdtsw)
!        call WRITEDIAGFI(ngrid,"zdtlw","LW heating","T s-1",3,zdtlw)

!     Output tracers
       if (tracer) then
        do iq=1,nq
!          call WRITEDIAGFI(ngrid,noms(iq),noms(iq),'kg/kg',3,zq(1,1,iq))
          call WRITEDIAGFI(ngridmx,trim(noms(iq))//'_surf',
     &                    trim(noms(iq))//'_surf',
     &                    'kg/kg',2,qsurf(1,iq) )

          call WRITEDIAGFI(ngridmx,trim(noms(iq))//'_col',
     &                    trim(noms(iq))//'_col',
     &                    'kg m^-2',2,qcol(1,iq) )



          if(waterrain)then
             CALL WRITEDIAGFI(ngridmx,"rain",
     &            "total precipitation","kg m-2 s-1",2,rainout)
          endif

!          call WRITEDIAGFI(ngridmx,trim(noms(iq))//'_aero',
!     &                    trim(noms(iq))//'_aero',
!     &                    'kg/kg',3,aerosol(1,1,iq))

          call WRITEDIAGFI(ngridmx,"tau_col",
     &         "Total aerosol optical depth","[]",2,tau_col)

!          call WRITEDIAGFI(ngridmx,trim(noms(iq))//'_reff',
!     &                    trim(noms(iq))//'_reff',
!     &                    'm',3,reffrad(1,1,iq))

        enddo
       endif

      ELSE         ! if(ngrid.eq.1)


!      ----------------------------------------------------------------------
!      Output in grads file "g1d" (ONLY when using testphys1d)
!      ----------------------------------------------------------------------

         if(countG1D.eq.saveG1D)then
         
!         call WRITEDIAGFI(ngrid,"tsurf","Surface temperature", 
!     &                  "K",0,tsurf)
         CALL writeg1d(ngrid,1,tsurf,'tsurf','K')
         CALL writeg1d(ngrid,1,ps,'ps','Pa')
         CALL writeg1d(ngrid,nlayer,zt,'T','K')
         CALL writeg1d(ngrid,nlayer,pq(1,1,1),'q','kg/kg')
         CALL writeg1d(ngrid,nlayer,aerosol,'aerosol','SI')
         CALL writeg1d(ngrid,nlayer,reffrad,'reffrad','SI')
         CALL writeg1d(ngrid,nlayer,zdtlw,'dtlw','SI')
         CALL writeg1d(ngrid,nlayer,zdtsw,'dtsw','SI')
         CALL writeg1d(ngrid,nlayer,zdtdyn,'dtdyn','SI')

!        radiation balance (optional)
         CALL writeg1d(ngrid,1,flux3/totarea,'ISR','W m-2')
         CALL writeg1d(ngrid,1,flux1/totarea,'ASR','W m-2')
         CALL writeg1d(ngrid,1,flux2/totarea,'OLR','W m-2')   

         if(tracer) then
            do iq=1,nq
              CALL writeg1d(ngrid,1,qsurf(1,iq),
     &              trim(noms(iq))//'_s','kg m^-2') 
              CALL writeg1d(ngrid,1,qcol(1,iq),
     &              trim(noms(iq))//'_c','kg m^-2') 
              CALL writeg1d(ngrid,nlayer,zq(1,1,iq),noms(iq),'kg/kg') 
            end do

            if(waterrain)then
               CALL writeg1d(ngrid,1,rainout,
     &              'rainfall','kg m-2 s-1')
            endif
         end if

         countG1D=1
         else
            countG1D=countG1D+1
         endif ! if time to save

      END IF       ! if(ngrid.ne.1)

      icount=icount+1

      RETURN
      END
